<?php

declare(strict_types=1);

namespace Drupal\Tests\metatag_ui_enhancements\FunctionalJavascript;

/**
 * Functional Javascript test for Metatag UI Enhancements.
 *
 * @group metatag_ui_enhancements
 */
class MetatagUiEnhancementsIntegrationTest extends MetatagUiEnhancementsTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalPlaceBlock('system_breadcrumb_block');
    $this->drupalPlaceBlock('local_tasks_block');
    $this->drupalPlaceBlock('page_title_block');

    $this->drupalCreateContentType(['type' => 'article']);
    $page = $this->getSession()->getPage();

    // Create a user who can add media fields.
    $user = $this->drupalCreateUser([
      'access administration pages',
      'administer node fields',
      'administer node form display',
      'administer user fields',
      'view media',
      'access media overview',
      'create media',
      'delete any media',
      'update any media',
      'bypass node access',
      'access content',
      'administer nodes',
      'edit any article content',
      'create article content',
      'administer blocks',
      'view the administration theme',
      'access content',
      'access content overview',
      'administer content types',
      'edit any article content',
      'delete any article content',
      'view published skos concept entities',
    ], NULL, TRUE);
    $this->drupalLogin($user);

    $this->drupalGet('/admin/structure/types/manage/article/fields/add-field');
    $page->find('css', "[name='new_storage_type'][value='metatag']")->getParent()->click();
    $page->pressButton('Continue');
    $page->fillField('label', 'Metatags');
    $this->waitForText('field_metatag');
    $page->pressButton('Continue');
    $page->pressButton('Save settings');
    $this->setUpSparql();
    $this->uploadMediaImage();
  }

  /**
   * Test adding default images in metatag enhancement UI.
   */
  public function testMetatagDefaultImagesIntegration() {
    $page = $this->getSession()->getPage();
    $assert_session = $this->assertSession();

    $this->drupalGet('admin/config/search/metatag/enhancement-ui/add-default-images');
    $assert_session->elementExists('css', '#edit-open-graph')->click();
    $this->assertElementExistsAfterWait('css', "#edit-open-graph-og-image-id-media-library-open-button")
      ->click();
    $this->waitForText('Add or select media');
    $assert_session->elementExists('css', '.js-media-library-item')->click();
    $assert_session->pageTextContains('1 of 1 item selected');
    $assert_session->elementExists('css', '.ui-dialog-buttonpane')->pressButton('Insert selected');
    $page->pressButton('Save');
    $assert_session->pageTextContains('Default images for metatags has been saved');

    $this->drupalGet('node/add/article');
    $page->fillField('title[0][value]', 'My page');
    $page->fillField('oe_content_content_owner[0][target_id]', 'Arab Common Market (http://publications.europa.eu/resource/authority/corporate-body/ACM)');

    $page->pressButton('Save');
    $this->drupalGet('node/1');
    $metas = $page->findAll('css', 'meta');
    $property = [];
    $content = [];

    foreach ($metas as $meta) {
      $property[] = $meta->getAttribute('property');
      $content[] = $meta->getAttribute('content');
    }
    $this->assertEquals('og:image', $property[1]);
    $this->assertStringContainsString('druplicon.png', $content[1], 'The page contains the expected substring.');
  }

  /**
   * Tests that the image uploaded appears as a metatag in the node.
   */
  public function testNodeImageUpload() {
    $assert_session = $this->assertSession();
    $session = $this->getSession();
    $page = $session->getPage();

    // Visit a node create page and open the media library.
    $this->drupalGet('node/add/article');
    $session->executeScript("jQuery('.vertical-tabs__menu-item')[2].children[0].click()");

    $this->assertElementExistsAfterWait('css', "#advanced_image_src_media_id-media-library-wrapper-field_metatags-0-advanced")
      ->pressButton('Add media');
    $this->waitForText('Add or select media');
    $assert_session->elementExists('css', '.js-media-library-item')->click();
    $assert_session->pageTextContains('1 of 1 item selected');
    $assert_session->elementExists('css', '.ui-dialog-buttonpane')->pressButton('Insert selected');
    $page->fillField('title[0][value]', 'My page');
    $page->fillField('oe_content_content_owner[0][target_id]', 'Arab Common Market (http://publications.europa.eu/resource/authority/corporate-body/ACM)');

    $page->pressButton('Save');
    $this->drupalGet('node/1');
    $links = $page->findAll('css', 'link');
    $hrefs = [];
    foreach ($links as $link) {
      $hrefs[] = $link->getAttribute('href');
      $rels[] = $link->getAttribute('rel');
    }

    $this->assertEquals('image_src', $rels[1]);
    $this->assertEquals('/web/core/misc/druplicon.png', $hrefs[1]);
  }

}
