<?php

declare(strict_types=1);

namespace Drupal\Tests\metatag_ui_enhancements\FunctionalJavascript;

use Drupal\file\Entity\File;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;
use Drupal\Tests\field_ui\Traits\FieldUiTestTrait;
use Drupal\Tests\sparql_entity_storage\Traits\SparqlConnectionTrait;

/**
 * Base class for functional tests of Media Library functionality.
 */
abstract class MetatagUiEnhancementsTestBase extends WebDriverTestBase {

  use FieldUiTestTrait;
  use SparqlConnectionTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'block',
    'field_ui',
    'field',
    'user',
    'node',
    'oe_content',
    'oe_content_page',
    'oe_corporate_site_info',
    'token',
    'token_module_test',
    'media',
    'media_library',
    'media_library_test',
    'hold_test',
    'metatag',
    'metatag_open_graph',
    'metatag_twitter_cards',
    'metatag_ui_enhancements',
    'metatag_favicons',
    'oe_media',
    'file',
    'system',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Asserts that text appears on page after a wait.
   *
   * @param string $text
   *   The text that should appear on the page.
   * @param int $timeout
   *   Timeout in milliseconds, defaults to 10000.
   *
   * @todo replace with whatever gets added in
   *   https://www.drupal.org/node/3061852
   */
  protected function waitForText($text, $timeout = 10000) {
    $result = $this->assertSession()->waitForText($text, $timeout);
    $this->assertNotEmpty($result, "\"$text\" not found");
  }

  /**
   * Waits for the specified selector and returns it if not empty.
   *
   * @param string $selector
   *   The selector engine name. See ElementInterface::findAll() for the
   *   supported selectors.
   * @param string|array $locator
   *   The selector locator.
   * @param int $timeout
   *   Timeout in milliseconds, defaults to 10000.
   *
   * @return \Behat\Mink\Element\NodeElement
   *   The page element node if found. If not found, the test fails.
   *
   * @todo replace with whatever gets added in
   *   https://www.drupal.org/node/3061852
   */
  protected function assertElementExistsAfterWait($selector, $locator, $timeout = 10000) {
    $element = $this->assertSession()->waitForElement($selector, $locator, $timeout);
    $this->assertNotEmpty($element);
    return $element;
  }

  /**
   * Upload a media image.
   */
  protected function uploadMediaImage() {
    $permanent_image = File::create([
      'uri' => 'core/misc/druplicon.png',
      'status' => 1,
    ]);
    $permanent_image->save();
    $media_storage = $this->container->get('entity_type.manager')->getStorage('media');

    $image_media = $media_storage->create([
      'bundle' => 'image',
      'name' => 'Image media title',
      'oe_media_image' => [
        [
          'target_id' => $permanent_image->id(),
          'alt' => 'default alt',
        ],
      ],
    ]);
    $image_media->save();
  }

}
